<?php
namespace Dfe\Moip\P;
use Dfe\Moip\API\Option;
// 2017-06-11
/** @method \Dfe\Moip\Method m() */
final class Charge extends \Df\StripeClone\P\Charge {
	/**
	 * 2017-06-12
	 * @override
	 * @see \Df\StripeClone\P\Charge::inverseCapture()
	 * @used-by \Df\StripeClone\P\Charge::request()
	 * @return bool
	 */
	protected function inverseCapture() {return true;}

	/**
	 * 2017-06-12
	 * @override
	 * @see \Df\StripeClone\P\Charge::k_Capture()
	 * @used-by \Df\StripeClone\P\Charge::request()
	 * @return string
	 */
	protected function k_Capture() {return 'delayCapture';}
	
	/**
	 * 2017-06-11 Ключ, значением которого является токен банковской карты.
	 * https://github.com/mage2pro/moip/blob/0.4.4/T/CaseT/Payment.php#L46-L49
	 * @override
	 * @see \Df\StripeClone\P\Charge::k_CardId()
	 * @used-by \Df\StripeClone\P\Charge::request()
	 * @return string
	 */
	function k_CardId() {return 'fundingInstrument';}

	/**
	 * 2017-07-15
	 * @override
	 * @see \Df\StripeClone\P\Charge::p()
	 * @used-by \Df\StripeClone\P\Charge::request()
	 * @return array(string => mixed)
	 */
	protected function p() {return [
		/**
		 * 2017-07-15
		 * «Número de parcelas.
		 * Válido para pagamentos por cartão.
		 * Se não for informado, o pagamento será realizado em 1 parcela.
		 * Mínimo 1 e Máximo 12.»
		 * https://dev.moip.com.br/v2.0/reference#criar-pagamento
		 */
		'installmentCount' => $this->m()->plan()
	];}

	/**
	 * 2017-06-11
	 * @override
	 * @see \Df\StripeClone\P\Charge::v_CardId()
	 * @used-by \Dfe\Moip\P\Reg::v_CardId()
	 * @used-by \Df\StripeClone\P\Charge::request()
	 * @param string $id
	 * @param bool $isPrevious [optional]
	 * @return array(string => mixed)
	 */
	function v_CardId($id, $isPrevious = false) {return [
		// 2017-06-09
		// «Credit Card data. It can be:
		// *) the ID of a credit card previously saved,
		// *) an encrypted credit card hash
		// *) the whole collection of credit card attributes (in case you have PCI DSS certificate).»
		// [Moip] The test bank cards: https://mage2.pro/t/3776
		//
		// hash: «Encrypted credit card data»
		// Conditional, String.
		// https://dev.moip.com.br/v2.0/docs/criptografia
		//
		// id: «Credit card ID.
		// This ID can be used in the future to create new payments. Internal reference.»
		// Conditional, String(16).
		//
		// 2017-06-13
		// A hash is a very long (345 symbols) base64-encoded string.
		// http://moip.github.io/moip-sdk-js
		// A card ID looks like «CRC-M423RWG3PK7J».
		'creditCard' => [$isPrevious ? 'id' : 'hash' => $id]
		// 2017-06-09
		// «Method used. Possible values: CREDIT_CARD, BOLETO, ONLINE_BANK_DEBIT, WALLET»
		// Required, String.
		,'method' => Option::BANK_CARD
	];}

	/**
	 * 2017-06-11
	 * https://github.com/mage2pro/moip/blob/0.4.2/T/CaseT/Payment.php#L50-L53
	 * @override
	 * @see \Df\StripeClone\P\Charge::k_DSD()
	 * @used-by \Df\StripeClone\P\Charge::request()
	 * @return string
	 */
	protected function k_DSD() {return 'statementDescriptor';}
}